from manifest_edit import BasePlugin
from manifest_edit import UTCTiming
import libfmp4


class Plugin(BasePlugin, UTCTiming):
    '''
    utc_add plugin.

    The purpose of this plugin is to add the "UTCTiming" element to a manifest,
    if not already present.

    For details on the "UTCTiming" element, see ISO/IEC 23009-1 5.3.1.2,
    5.3.1.3, 5.8.4.11 and 5.8.5.7.

    Several UTC Timing Scheme are possible, each requiring its own syntax for
    the value (see 5.8.5.7).

    You can specify the Scheme and Value that utc_add will add to the manifest
    by means of the .yml pipeline configuration file.

    "Scheme" can be specified using one of the following strings, the plugin
    will take care of applying the correct SchemeUriId string:

    - ntp
    - sntp
    - http-head
    - http-xsdate
    - http-iso
    - http-ntp
    - direct

    Each scheme requires to specify a value using a specific syntax. This plugin
    makes no effort to check that the provided "Value" string respect the
    correct syntax. The provided Value string will be used "as is".

    Example of configuration file using the "urn:mpeg:dash:utc:ntp:2014" scheme:
    - mpd:
        - manifest_edit.plugins.utc_add:
            ntp:
            https://time.akamai.com/?iso

    Limitations: only implemented for .mpd manifests
    '''

    def add(self, manifest):
        # add only works on manifests that do not already have an UTCTiming tag

        if not len(manifest.utc_timings):
            # check if the scheme to add is valid
            for scheme, value in self.config.items():
                if scheme not in self.SUPPORTED_UTC_SCHEMES:
                    print('Unsupported scheme {scheme}'.format(scheme=scheme))
                else:
                    manifest.utc_timings.append(
                        libfmp4.mpd.descriptor(
                            self.SUPPORTED_UTC_SCHEMES[scheme], value)
                    )

    def mpd(self, manifest):

        self.add(manifest)
